@@ -23,7 +23,7 @@ import re
 from restclient.rest import url_quote
 
 from couchdbkit.exceptions import InvalidAttachment
-from couchdbkit.resource import CouchdbResource, ResourceNotFound
+from couchdbkit.resource import CouchdbResource, ResourceNotFound, ResourceConflict
 from couchdbkit.utils import validate_dbname
 
 DEFAULT_UUID_BATCH_COUNT = 1000
@@ -54,6 +54,8 @@ class Server(object):
         self.res = CouchdbResource(uri, transport=transport)
         self.uuids = []
         
+    def __repr__(self):
+        return "<%s %s>" % (self.__class__.__name__, self.dbname)
         
     def info(self):
         """ info of server 
@@ -189,7 +191,7 @@ class Database(object):
         :return: dict, representation of CouchDB document as
          a dict.
         """
-        self.escape_docid(docid)
+        docid = self.escape_docid(docid)
         if rev is not None:
             doc = self.res.get(docid, rev=rev)
         else:
@@ -227,7 +229,7 @@ class Database(object):
         an additional field, _revs, the value being a list of 
         the available revision IDs. 
         """
-        self.escape_docid(docid)
+        docid = self.escape_docid(docid)
         try:
             if with_doc:
                 doc_with_rev = self.res.get(docid, revs=True)
@@ -258,7 +260,8 @@ class Database(object):
             doc['_attachments'] = self.encode_attachments(doc['_attachments'])
             
         if '_id' in doc:
-            self.escape_docid(doc['_id'])
+            doc['_id'] = self.escape_docid(doc['_id'])
+            print doc['_id']
             res = self.res.put(doc['_id'], payload=doc)
         else:
             try:
@@ -296,13 +299,12 @@ class Database(object):
             uuid_count = max(len(noids), self.server.uuid_batch_count)
             for doc in noids:
                 nextid = self.server.next_uuid(count=uuid_count)
-                print doc
                 if nextid:
                     doc['_id'] = nextid
                     
             # make sure we have a corret id
             for doc in ids:
-                self.escape_docid(doc['_id'])
+                doc['_id'] = self.escape_docid(doc['_id'])
                     
         payload = { "docs": docs }
         if all_or_nothing:
@@ -336,7 +338,7 @@ class Database(object):
             if not '_id' or not '_rev' in doc:
                 raise KeyError('_id and _rev are required to delete a doc')
                 
-            self.escape_docid(doc['_id'])
+            doc['_id'] = self.escape_docid(doc['_id'])
             result = self.res.delete(doc['_id'], rev=doc['_rev'])
         elif isinstance(doc, basestring): # we get a docid
             data = self.res.head(doc)
@@ -507,17 +509,16 @@ class Database(object):
     def __contains__(self, docid):
         return self.doc_exist(docid)
         
-    def __getitem__(self, id):
-        return self.get(id)
+    def __getitem__(self, docid):
+        return self.get(docid)
         
     def __setitem__(self, docid, doc):
-        res = self.res.put(docid, payload=doc)
-        doc.update({ '_id': res['id'], '_rev': res['rev']})
+        doc['_id'] = docid
+        self.save_doc(doc)
+        
         
     def __delitem__(self, docid):
-       data = self.res.head(docid)
-       result = self.res.delete(docid, 
-               rev=self.res.get_response()['etag'].strip('"'))
+       self.delete_doc(docid)
 
     def __iter__(self):
         return self.iterdocuments()
@@ -526,9 +527,9 @@ class Database(object):
         return (len(self) > 0)
         
     def escape_docid(self, docid):
-        if docid.startswith('_design/'):
-            docid = "_design/%s" % (cgi.escape(docid[8:]))
-            
+        docid = url_quote(docid, safe=":/")
+        return docid  
+
     def encode_attachments(self, attachments):
         for k, v in attachments.iteritems():
             if v.get('stub', False):
@@ -559,15 +560,13 @@ class ViewResults(object):
 
     def iterator(self):
         self._fetch_if_needed()
-        rows = self._result_cache.get('rows')
-        if not rows:
-            yield {}
-        else:
-            for row in rows:
-                if self.view._wrapper is not None:
-                    yield self.view._wrapper(row)
-                else:
-                    yield row
+        rows = self._result_cache.get('rows', [])
+        wrapper = self.view._wrapper
+        for row in rows:
+            if  wrapper is not None:
+                yield self.view._wrapper(row)
+            else:
+                yield row
                     
     def one(self):
         return list(self)[0]
@@ -620,7 +619,7 @@ class ViewResults(object):
         return self.count()
 
     def __nonzero__(self):
-        bool(len(self))
+        return bool(len(self))
         
         
 class ViewInterface(object):
@@ -632,6 +631,9 @@ class ViewInterface(object):
     def __call__(self, **params):
         return ViewResults(self, **params)
         
+    def __iter__(self):
+        return self()
+        
     def _exec(self, **params):
         raise NotImplementedError
         
@@ -641,6 +643,7 @@ class View(ViewInterface):
         ViewInterface.__init__(self, db, wrapper=wrapper)
         self.view_path = view_path
         
+        
     def _exec(self, **params):
         if 'keys' in params:
             return self._db.res.post(self.view_path, **params)
